"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BettererCoverageTest = void 0;
const betterer_1 = require("@betterer/betterer");
const constraint_1 = require("./constraint");
const differ_1 = require("./differ");
const goal_1 = require("./goal");
const serialiser_1 = require("./serialiser");
const utils_1 = require("./utils");
/**
 * @public `BettererCoverageTest` provides a wrapper around {@link @betterer/betterer#BettererTest | `BettererTest` }
 * that makes it easier to implement coverage tests.
 *
 * @remarks `BettererCoverageTest` provides a useful example for the more complex possibilities of the {@link @betterer/betterer#BettererTestOptions | `BettererTestOptions` }
 * interface.
 */
class BettererCoverageTest extends betterer_1.BettererTest {
    /**
     * @internal This could change at any point! Please don't use!
     *
     * You should not construct a `BettererCoverageTest` directly.
     */
    constructor(test, coverageSummaryPath = './coverage/coverage-summary.json') {
        super({
            test: (run) => test(run, coverageSummaryPath, this._included, this._excluded),
            constraint: constraint_1.constraint,
            differ: differ_1.differ,
            goal: goal_1.goal,
            serialiser: {
                serialise: serialiser_1.serialise,
                deserialise: serialiser_1.deserialise
            }
        });
        this._excluded = [];
        this._included = [];
    }
    /**
     * Add a list of {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Regular_Expressions | Regular Expression } filters for files to exclude when running the test.
     *
     * @param excludePatterns - RegExp filters to match file paths that should be excluded.
     * @returns This {@link @betterer/coverage#BettererCoverageTest | `BettererCoverageTest`}, so it is chainable.
     */
    exclude(...excludePatterns) {
        if (!this._included.length) {
            this.include('**/*');
        }
        this._excluded = [...this._excluded, ...(0, utils_1.flatten)(excludePatterns)];
        return this;
    }
    /**
     * Add a list of {@link https://www.npmjs.com/package/glob#user-content-glob-primer | glob }
     * patterns for files to include when running the test.
     *
     * @param includePatterns - Glob patterns to match file paths that should be included. All
     * `includes` should be relative to the {@link https://phenomnomnominal.github.io/betterer/docs/test-definition-file | test definition file}.
     * @returns This {@link @betterer/coverage#BettererCoverageTest | `BettererCoverageTest`}, so it is chainable.
     */
    include(...includePatterns) {
        this._included = [...this._included, ...(0, utils_1.flatten)(includePatterns)];
        return this;
    }
}
exports.BettererCoverageTest = BettererCoverageTest;
//# sourceMappingURL=coverage-test.js.map