"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.testTotal = exports.test = void 0;
const tslib_1 = require("tslib");
const errors_1 = require("@betterer/errors");
const fs_1 = require("fs");
const minimatch_1 = (0, tslib_1.__importDefault)(require("minimatch"));
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const utils_1 = require("./utils");
async function test(run, relativeCoverageSummaryPath, included, excluded) {
    const baseDirectory = getTestBaseDirectory(run);
    const absoluteCoverageSummaryPath = path_1.default.resolve(baseDirectory, relativeCoverageSummaryPath);
    const summary = await readCoverageSummary(absoluteCoverageSummaryPath);
    delete summary.total;
    const uncovered = {};
    Object.keys(summary)
        .filter((filePath) => !included.length || included.some((include) => (0, minimatch_1.default)(filePath, include)))
        .filter((filePath) => !excluded.length || !excluded.some((exclude) => exclude.test(filePath)))
        .forEach((filePath) => {
        const fileCoverage = summary[filePath];
        const relativeFilePath = (0, utils_1.normalisedPath)(path_1.default.relative(baseDirectory, filePath));
        uncovered[relativeFilePath] = getUncoveredIssues(fileCoverage);
    });
    return uncovered;
}
exports.test = test;
async function testTotal(run, relativeCoverageSummaryPath) {
    const baseDirectory = getTestBaseDirectory(run);
    const absoluteCoverageSummaryPath = path_1.default.resolve(baseDirectory, relativeCoverageSummaryPath);
    const { total } = await readCoverageSummary(absoluteCoverageSummaryPath);
    return {
        total: getUncoveredIssues(total)
    };
}
exports.testTotal = testTotal;
async function readCoverageSummary(coverageSummaryPath) {
    const result = await loadCoverageFile(coverageSummaryPath);
    if (result) {
        return result;
    }
    else {
        throw new errors_1.BettererError(`Could not read coverage summary`);
    }
}
async function loadCoverageFile(coverageSummaryPath) {
    try {
        const coverageReport = await fs_1.promises.readFile(coverageSummaryPath, 'utf-8');
        return decodeCoverageSummary(JSON.parse(coverageReport));
    }
    catch (e) {
        return null;
    }
}
function decodeCoverageSummary(data) {
    if (isIstanbulSummary(data) && data.total != null) {
        return data;
    }
    return null;
}
function isIstanbulSummary(data) {
    const maybeSummary = data;
    return maybeSummary && Object.keys(maybeSummary).every((key) => isFileCoverage(maybeSummary[key]));
}
function isFileCoverage(data) {
    const maybeCoverage = data;
    return (maybeCoverage &&
        isCoverage(maybeCoverage.branches) &&
        isCoverage(maybeCoverage.functions) &&
        isCoverage(maybeCoverage.lines) &&
        isCoverage(maybeCoverage.statements));
}
function isCoverage(data) {
    const maybeData = data;
    return (maybeData &&
        (0, utils_1.isNumber)(maybeData.covered) &&
        (0, utils_1.isNumber)(maybeData.pct) &&
        (0, utils_1.isNumber)(maybeData.skipped) &&
        (0, utils_1.isNumber)(maybeData.total));
}
function getUncoveredIssues(fileCoverage) {
    return {
        lines: fileCoverage.lines.total - fileCoverage.lines.covered,
        statements: fileCoverage.statements.total - fileCoverage.statements.covered,
        functions: fileCoverage.functions.total - fileCoverage.functions.covered,
        branches: fileCoverage.branches.total - fileCoverage.branches.covered
    };
}
function getTestBaseDirectory(run) {
    return path_1.default.dirname(run.test.configPath);
}
//# sourceMappingURL=test.js.map